/*Example created by Etienne Desrousseaux on the 12/03/2017
 * 
 * This example connects an arduino to a remote, downloads data for a single led connected to pin 3, and send a confirmation to the remote
 * 
 * You can create your own remote on http://www.etienne-desrousseaux.com/e-duino/index.php using the code hackaday.
 * 
 * Please, check out the project page at https://hackaday.io/project/20088-e-duino
 * 
*/
#include <SPI.h>
#include <Ethernet.h>

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };

char server[] = "www.etienne-desrousseaux.com";//the server where is the system for now

IPAddress ip(192, 168, 0, 177);

EthernetClient client;

void setup() {
  Serial.begin(9600);//debugging serial
  Serial.println("serial begun");
  
  if (Ethernet.begin(mac) == 0) {
    Serial.println("error while connecting");
    // try to congifure using IP address instead of DHCP:
    Ethernet.begin(mac, ip);
  }
  pinMode(3, OUTPUT);

  delay(10500);//leave a while for ethernet to connect, needed in my case to avoid a hell of "not connected" on serial later on
  Serial.println("connected...");
}

bool lastB;

void loop() {
  bool b = actualizeSwitch();
  if(b != lastB)
  {
    lastB = b;
    if(b)
    {
      confirmToServer(String("led turned on"));
    }
    else
    {
      confirmToServer(String("led turned off"));
    }
  }
}


void confirmToServer(String toConfirm)
{
  Serial.println("connecting...");
  if (client.connect(server, 80)) //requests the value of the input named 'switch_test' of the 'test' remote
  {
    Serial.println("connected");
    Serial.print("sending value : ");
    toConfirm.replace(" ", "%20");
    // Make a HTTP request:
    String str = String("GET /e-duino/saveRemoteSingleValue.php?remoteId=test&password=password&input=feedback_test&value=") +  
    toConfirm + String(" HTTP/1.1");
    Serial.println(str);
    client.println(str);
    client.println("Host: www.etienne-desrousseaux.com");
    client.println("Connection: close");
    client.println();

    while(!client.available())
    {
      delay(3);
    }
    Serial.println("confirmation sent");
    while(client.available())
    {
      Serial.print((char)client.read());
    }
  }
  else
  {
    Serial.println("not connected");
    if(client.connected())
    {
      Serial.println("still connected");
    }
  }
}

bool actualizeSwitch() {
  bool boo = false;

  Serial.println("connecting...");
  if (client.connect(server, 80)) //requests the value of the input named 'switch_test' of the 'test' remote
  {
    Serial.println("connected");
    // Make a HTTP request:
    client.println("GET /e-duino/getRemoteValue.php?remoteId=test&input=switch_test HTTP/1.1");
    client.println("Host: www.etienne-desrousseaux.com");
    client.println("Connection: close");
    client.println();

    while(!client.available())
    {
      delay(3);
    }
  }
  else
  {
    Serial.println("not connected");
    if(client.connected())
    {
      Serial.println("still connected");
    }
    if(lastB)
    {
      digitalWrite(3, HIGH);
      boo = true;
    }
    else
    {
      digitalWrite(3, LOW);
      boo = false;
    }
  }
  if(client.available())
  {
    Serial.println("reading content...");
    char endOfHeaders[] = "\r\n\r\n";
  
    client.setTimeout(10000);
    bool ok = client.find(endOfHeaders);//skip html data
  
    if(ok)
    {
      Serial.println("found content");
      for(int i = 0; i < 3; i++)//skip unwanted characters
      {
        if(client.available())
        {
          client.read();
        }
      }
      byte b = 0;
      char goal[] = "true";
      while(client.available())//this system can be used for short strings too, since it test characters one by one.
      {
        char c = client.read();
        Serial.println(c);
        if(c == goal[b])
        {
          b++;
          if(b >= 4)//if 4 good characters, then the value starts with 'true', so we're good
          {
            break;
          }
        }
        else//wrong character, we return to 0 and stop there
        {
          b = 0;
          break;
        }
      }
      if(b > 0)
      {
        digitalWrite(3, HIGH);
        boo = true;
      }
      else
      {
        digitalWrite(3, LOW);
        boo = false;
      }
    }
    else//bad html signature
    {
      Serial.println("bad format data received");
      if(lastB)
      {
        digitalWrite(3, HIGH);
        boo = true;
      }
      else
      {
        digitalWrite(3, LOW);
        boo = false;
      }
    }
  }
  client.stop();

  return boo;
}
